﻿* Encoding: UTF-8.



* ******************************************************************************** .
*                         SUPPLEMENTAL MATERIALS FOR                        .
*              MULTIVARIATE DATA ANALYSIS -- EIGHTH EDITION         .
*                      HAIR, BLACK, BABIN AND ANDERSON                     .
*                                                                                                      .
*        CHAPTER 2 -- EXAMINING YOUR DATA AND MISSING_DATA  .
*                                        SPSS SYNTAX                                       .
* ******************************************************************************** .



* ********************************************.
* PART A -- EXAMINING YOUR DATA .
* ******************************************** .

GET FILE = 'HBAT.SAV' .


* *************************************************** .
 
* BASIC DESCRIPTIVE STATISTICS AND GRAPHICS .

* EXAMPLE -- X6 .

TITLE 'BASIC DESCRIPTIVE STATISTICS AND GRAPHICS -- X6' .

EXAMINE
  VARIABLES=x6
  /PLOT  STEMLEAF HISTOGRAM NPPLOT
  /STATISTICS DESCRIPTIVES
  /CINTERVAL 95
  /MISSING LISTWISE
  /NOTOTAL.

* EXAMPLE -- X6 BY X1 .
TITLE 'BASIC DESCRIPTIVE STATISTICS AND GRAPHICS -- X6 BY X1' .
EXAMINE
  VARIABLES=x6 BY x1 /PLOT=BOXPLOT/STATISTICS=NONE/NOTOTAL
  /MISSING=REPORT.
EXAMINE
  VARIABLES=x7 BY x1 /PLOT=BOXPLOT/STATISTICS=NONE/NOTOTAL
  /MISSING=REPORT.
ONEWAY
  x6 x7 BY x1
  /MISSING ANALYSIS
  /POSTHOC = SCHEFFE ALPHA(.05).




* ************************************************************************.
* PART B -- EXAMINATION AND REMEDY OF MISSING DATA .
* ************************************************************************ .


TITLE 'EXAMINATION AND REMEDY OF MISSING DATA -- INITIAL ANALYSIS' .

GET FILE ='HBAT_MISSING.SAV' .

COUNT NMISS = V1 TO V14 (MISSING) .
FREQ NMISS .


* USE MISSING VALUE ANALYSIS (MVA) MODULE IN SPSS .

MVA
  v1 v2 v3 v4 v5 v6 v7 v8 v9  v10 v11 v12 v13 v14
  /MAXCAT = 25
  /ID = id
  /CATEGORICAL = v10 v11 v12 v13 v14
  /TTEST PROB PERCENT=5
  /DPATTERN DESCRIBE=v1 v2 v3 v4 v5 v6 v7 v8 v9
  /MPATTERN NOSORT DESCRIBE=v1 v2 v3 v4 v5 v6 v7 v8 v9
  /TPATTERN NOSORT PERCENT=1 DESCRIBE=v1 v2 v3 v4 v5 v6 v7 v8 v9
  /LISTWISE
  /PAIRWISE
  /EM ( TOLERANCE=0.001 CONVERGENCE=0.0001 ITERATIONS=100 )
  /REGRESSION ( TOLERANCE=0.001 FLIMIT=4.0 NPREDICTORS= 3 ADDTYPE=RESIDUAL ) .


* Delete cases with greater than 50% missing .

TITLE 'MISSING DATA -- DELETE CASES WITH GT 50 PERCENT MISSING' .

GET FILE ='HBAT_MISSING.SAV' .

COUNT NMISS = V1 TO V14 (MISSING) .
SELECT IF (NMISS LE 6) .
FREQ NMISS .

TITLE 'MVA OF 64 CASES WITH V1' .
MVA
  v1 v2 v3 v4 v5 v6 v7 v8 v9  v10 v11 v12 v13 v14
  /MAXCAT = 25
  /ID = id
  /CATEGORICAL = v10 v11 v12 v13 v14
  /TTEST PROB PERCENT=5
  /DPATTERN DESCRIBE=v1 v2 v3 v4 v5 v6 v7 v8 v9
  /MPATTERN NOSORT DESCRIBE=v1 v2 v3 v4 v5 v6 v7 v8 v9
  /TPATTERN NOSORT PERCENT=1 DESCRIBE=v1 v2 v3 v4 v5 v6 v7 v8 v9
  /LISTWISE
  /PAIRWISE
  /EM ( TOLERANCE=0.001 CONVERGENCE=0.0001 ITERATIONS=100 )
  /REGRESSION ( TOLERANCE=0.001 FLIMIT=4.0 NPREDICTORS= 3 ADDTYPE=RESIDUAL )  .




* **************************************************************************** .
* Part 2 -- Analysis of Missing Data after variables (X1) and Cases deleted .
* **************************************************************************** .

* We recommend creating a new Missing Data file which eliminates V1 and the six cases identified in Part A .
* If not, delete cases from HBAT_MISSING file .


TITLE 'DELETE CASES GT 50 PERCENT MISSING AND DELETE V1' .

GET FILE ='HBAT_MISSING.SAV' .

COUNT NMISS = V2 TO V14 (MISSING) .
FREQ NMISS .
SELECT IF (NMISS LE 5) .
FREQ NMISS .

TITLE 'MVA OF 64 CASES AND V1 DELETED' .

MVA
  v2 v3 v4 v5 v6 v7 v8 v9  v10 v11 v12 v13 v14
  /MAXCAT = 25
  /ID = id
  /CATEGORICAL = v10 v11 v12 v13 v14
  /TTEST PROB PERCENT=5
  /MISMATCH NOSORT PERCENT=5
  /MPATTERN NOSORT DESCRIBE=v2 v3 v4 v5 v6 v7 v8 v9
  /TPATTERN NOSORT PERCENT=1 DESCRIBE=v2 v3 v4 v5 v6 v7 v8 v9
  /LISTWISE
  /PAIRWISE
  /EM ( TOLERANCE=0.001 CONVERGENCE=0.0001 ITERATIONS=100 )
  /REGRESSION ( TOLERANCE=0.001 FLIMIT=4.0 NPREDICTORS= 4 ADDTYPE=RESIDUAL )  .




* IMPUTE MEAN SUBSTITUTION .

TITLE 'IMPUTE MEAN SUBSTITUTION  ' .
RMV /v2_MEANSUB=SMEAN(v2) /v3_MEANSUB=SMEAN(v3) /V4_MEANSUB=SMEAN(V4) /V5_MEANSUB=SMEAN(V5) /V6_MEANSUB=SMEAN(V6) /V7_MEANSUB=SMEAN(V7) /V8_MEANSUB=SMEAN(V8) /V9_MEANSUB=SMEAN(V9) .
EXECUTE .


* ************************************************************************ .
* IMPUTATION OF LISTWISE, PAIRWISE and EM with Missing Value Procedure .
* ************************************************************************ .

TITLE 'IMPUTATION OF LISTWISE, PAIRWISE and EM with MVA' .

MVA VARIABLES=v2 v3 v4 v5 v6 v7 v8 v9 
  /LISTWISE
  /PAIRWISE
  /EM(TOLERANCE=0.001 CONVERGENCE=0.0001 ITERATIONS=25) .



* ************************************************************************ .
* IMPUTATION WITH REGRESSION MODEL WITHOUT RESIDUALS .
* ************************************************************************ .
TITLE 'IMPUTATION WITH REGRESSION MODEL WITHOUT RESIDUALS' .
MVA VARIABLES=v2 v3 v4 v5 v6 v7 v8 v9 
  /REGRESSION(TOLERANCE=0.001 FLIMIT=4.0 ADDTYPE=NONE ).



* ************************************************************************ .
* IMPUTATION WITH REGRESSION MODEL -- OPTION 1  .
*     Error terms are chosen randomly from the observed residuals of complete cases to be added to the regression estimates .
* ************************************************************************ .

TITLE 'IMPUTATION WITH REGRESSION MODEL -- OPTION 1' .
SUBTITLE 'RESULTS WILL VARY SLIGHTLY DUE TO RANDOM COMPONENTS' .

MVA VARIABLES=v2 v3 v4 v5 v6 v7 v8 v9 
  /REGRESSION(TOLERANCE=0.001 FLIMIT=4.0 ADDTYPE=RESIDUAL).


* ************************************************************************ .
* IMPUTATION WITH REGRESSION MODEL -- OPTION 2  .
*      Error terms are randomly drawn from a distribution with the expected value 0 and the standard deviation . 
*      equal to the square root of the mean squared error term (sometimes called the root mean squared error, or RMSE) of the regression.
* ************************************************************************ .

TITLE 'IMPUTATION WITH REGRESSION MODEL -- OPTION 2' .
SUBTITLE 'RESULTS WILL VARY SLIGHTLY DUE TO RANDOM COMPONENTS' .

MVA VARIABLES=v2 v3 v4 v5 v6 v7 v8 v9 
  /REGRESSION(TOLERANCE=0.001 FLIMIT=4.0 ADDTYPE=NORMAL).

* ************************************************************************ .
* IMPUTATION WITH REGRESSION MODEL -- OPTION 3  .
*      Error terms are randomly drawn from the t(n) distribution and scaled by the RMSE. 
*      The degrees of freedom can be specified in parentheses. 
*      If T is specified without a value, the default degrees of freedom is 5.
* ************************************************************************ .

TITLE 'IMPUTATION WITH REGRESSION MODEL -- OPTION 3' .
SUBTITLE 'RESULTS WILL VARY SLIGHTLY DUE TO RANDOM COMPONENTS' .

MVA VARIABLES=v2 v3 v4 v5 v6 v7 v8 v9 
  /REGRESSION(TOLERANCE=0.001 FLIMIT=4.0 ADDTYPE=T).


* ************************************************************************ .
* IMPUTATION WITH MULTIPLE IMPUTATION EMPLOYING AUXILLIARY VARIABLES  .
*     NOTE: The OUTFILE subcommand is required unless imputation is turned off (/IMPUTE METHOD=NONE). 
* ************************************************************************ .

TITLE ' MULTIPLE IMPUTATION WITH AUXILLIARY VARIABLES' .
SUBTITLE 'RESULTS WILL VARY SLIGHTLY DUE TO RANDOM COMPONENTS' .

MULTIPLE IMPUTATION v2 v3 v4 v5 v6 v7 v8 v9 v10 v11 v12 v13 v14 
  /IMPUTE METHOD=FCS MAXITER= 20 NIMPUTATIONS=5 SCALEMODEL=LINEAR INTERACTIONS=NONE SINGULAR=1E-012 
    MAXPCTMISSING=NONE  MAXCASEDRAWS=50 MAXPARAMDRAWS=2
  /CONSTRAINTS v2( MIN=.01 MAX=10)
  /CONSTRAINTS v3( MIN=.01 MAX=10)
  /CONSTRAINTS v4( MIN=.01 MAX=10)
  /CONSTRAINTS v5( MIN=.01 MAX=10)
  /CONSTRAINTS v6( MIN=.01 MAX=10)
  /CONSTRAINTS v7( MIN=.01 MAX=10)
  /CONSTRAINTS v8( MIN=.01 MAX=100)
  /CONSTRAINTS v9( MIN=.01 MAX=10)
  /MISSINGSUMMARIES  OVERALL  PATTERNS
  /IMPUTATIONSUMMARIES MODELS DESCRIPTIVES 
     /OUTFILE  IMPUTATIONS= 'MI_WITH_AUX' .




* ************************************************************************ .
* MULTIPLE IMPUTATION WITHOUT AUXILLIARY VARIABLES  .
*     ADDITION OF MISSING DATA PATTERNS AND OVERALL STATISTICS .
* ************************************************************************ .
TITLE 'MULTIPLE IMPUTATION WITHOUT AUXILLIARY VARIABLES' .
SUBTITLE 'RESULTS WILL VARY SLIGHTLY DUE TO RANDOM COMPONENTS' .

MULTIPLE IMPUTATION v2 v3 v4 v5 v6 v7 v8 v9 
  /IMPUTE METHOD=FCS MAXITER= 20 NIMPUTATIONS=5 SCALEMODEL=LINEAR INTERACTIONS=NONE SINGULAR=1E-012 
    MAXPCTMISSING=NONE  MAXCASEDRAWS=50 MAXPARAMDRAWS=2
  /CONSTRAINTS v2( MIN=.01 MAX=10)
  /CONSTRAINTS v3( MIN=.01 MAX=10)
  /CONSTRAINTS v4( MIN=.01 MAX=10)
  /CONSTRAINTS v5( MIN=.01 MAX=10)
  /CONSTRAINTS v6( MIN=.01 MAX=10)
  /CONSTRAINTS v7( MIN=.01 MAX=10)
  /CONSTRAINTS v8( MIN=.01 MAX=100)
  /CONSTRAINTS v9( MIN=.01 MAX=10)
  /MISSINGSUMMARIES OVERALL  PATTERNS
  /IMPUTATIONSUMMARIES MODELS DESCRIPTIVES 
      /OUTFILE  IMPUTATIONS = 'MI_WITHOUT_AUX' .



