﻿



* ******************************************************************************** .
*                         SUPPLEMENTAL MATERIALS FOR                        .
*              MULTIVARIATE DATA ANALYSIS -- EIGHTH EDITION         .
*                      HAIR, BLACK, BABIN AND ANDERSON                     .
*                                                                                                      .
*                        CHAPTER 4 -- CLUSTER ANALYSIS                        .
*                                        SPSS SYNTAX                                       .
* ******************************************************************************** .

* ************************************************************************************* .
*  NOTE -- THE SPSS OUTPUT IS SUPPLEMENTED BY SAS OUTPUT    .
*               AND PROVIDES ADDITIONAL METRICS                                 .
*                  -- CCC                                                                               .
*                  -- PSEUDO F                                                                     .
*                  -- PSEUDO T-SQUARED                                                    .
*               PLEASE REFER TO THE ASSOCIATED SAS SYNTAX             .
*               AND OUTPUT FILES FOR THIS MATERIAL                              .
* ************************************************************************************* .

* ********************* .
* READ HBAT FILE  .
* ********************* .
TITLE 'CLUSTER ANALYSIS' .
GET FILE = 'HBAT.SAV' .


* ******************************************************** .
* INITIAL HIERARCHICAL CLUSTER ANALYSIS   .
*   -- 100 CASES                                                 .
*   -- VARIABLES X6 X8 X12 X15 X18                    .
* ******************************************************** .
TITLE 'HIERARCHICAL CLUSTER ANALYSIS' .
SUBTITLE 'INITIAL 100 CASES' .

CLUSTER  x6 x8 x12 x15 x18
  /METHOD WARD
  /MEASURE= SEUCLID
  /PRINT SCHEDULE
  /PLOT VICICLE.

* ********************************************** .
* DELETE TWO CASES DESIGNATED   .
* AS OUTLIERS -- CASES 6 and 87         .
* ********************************************** .


SELECT IF ((ID NE 6) AND (ID NE 87) ) .
EXECUTE .


* ********************************************** .
* HIERARCHICAL CLUSTER ANALYSIS   .
*   -- 98 REMAINING CASES                    .
* *********************************************** .
SUBTITLE 'ANALYSIS WITH TWO OUTLIERS DELETED' .

CLUSTER  x6 x8 x12 x15 x18
  /METHOD WARD
  /MEASURE= SEUCLID
  /PRINT SCHEDULE
  /PLOT VICICLE
  /SAVE CLUSTER(4) .

* ************************************************************************* .
* COMPUTE MEAN-CENTERED VARIABLES FOR PROFILING  .
* ************************************************************************* .

COMPUTE X6C=x6 - 7.81. 
COMPUTE X8C=x8-5.37. 
COMPUTE X12C=x12-5.12. 
COMPUTE X15C=x15-5.15. 
COMPUTE X18C=x18-3.89. 
EXECUTE.


* *************************************************** .
* PROFILING WITH CENTERED VARIABLES  .
* *************************************************** .
SUBTITLE 'PROFILING 4 CLUSTERS WITH CENTERED VARIABLES' .
MEANS
  TABLES=x6c x8c x12c x15c x18c  BY CLU4_1
  /CELLS MEAN COUNT   .



* *************************************************** .
* NONHIERARCHICAL CLUSTER ANALYSIS .
*   -- 4 CLUSTER SOLUTION                         .
*   -- 98 REMAINING CASES                    .
* *************************************************** .

TITLE 'NONHIERARCHICAL 4 CLUSTER SOLUTION' .

SUBTITLE '4 CLUSTER SOLUTION' .  
QUICK CLUSTER
  x6 x8 x12 x15 x18
  /MISSING=LISTWISE
  /CRITERIA= CLUSTER(4) MXITER(100) CONVERGE(0)
  /METHOD=KMEANS(NOUPDATE)
  /SAVE CLUSTER
  /PRINT INITIAL ANOVA.

* *************************************************** .
* PROFILING WITH CENTERED VARIABLES  .
* *************************************************** .
SUBTITLE ' PROFILING 4 CLUSTERS WITH CENTERED VARIABLES' .

MEANS
  TABLES=x6c x8c x12c x15c x18c  BY QCL_1
  /CELLS MEAN COUNT   .

* **************************************************** .
* VALIDATING WITH OUTCOME VARIABLES  .
* **************************************************** .

SUBTITLE 'VALIDATING 4 CLUSTERS WITH OUTCOME VARIABLES' .

MEANS
  TABLES=x19 x20 x21 x22  BY QCL_1
  /CELLS MEAN COUNT
  /STATISTICS ANOVA .


* ************************************************************** .
* PROFILING WITH CUSTOMER CHARACTERISTICS  .
* ************************************************************** .

SUBTITLE 'PROFILING 4 CLUSTERS WITH CUSTOMER CHARACTERISTICS' .

CROSSTABS TAB = X1, X2, X3, X4, X5 BY QCL_1 .



* *************************************************** .
* NONHIERARCHICAL CLUSTER ANALYSIS .
*   -- 3 CLUSTER SOLUTION                         .
*   -- 98 REMAINING CASES                    .
* *************************************************** .
TITLE 'NONHIERARCHICAL 3 CLUSTER SOLUTION' .

SUBTITLE '3 CLUSTER SOLUTION' .  
  
  QUICK CLUSTER
  x6 x8 x12 x15 x18
  /MISSING=LISTWISE
  /CRITERIA= CLUSTER(3) MXITER(100) CONVERGE(0)
  /METHOD=KMEANS(NOUPDATE)
  /SAVE CLUSTER
  /PRINT INITIAL ANOVA.

* *************************************************** .
* PROFILING WITH CENTERED VARIABLES  .
* *************************************************** .

SUBTITLE ' PROFILING 3 CLUSTERS WITH CENTERED VARIABLES' .

MEANS
  TABLES=x6c x8c x12c x15c x18c  BY QCL_2
  /CELLS MEAN COUNT   .

* ************************************************************** .
* PROFILING WITH CUSTOMER CHARACTERISTICS  .
* ************************************************************** .
SUBTITLE 'PROFILING 3 CLUSTERS WITH CUSTOMER CHARACTERISTICS' .

CROSSTABS TAB = X1, X2, X3, X4, X5 BY QCL_2 .

