* Encoding: UTF-8.


* ******************************************************************************** .
*                         SUPPLEMENTAL MATERIALS FOR                        .
*              MULTIVARIATE DATA ANALYSIS -- EIGHTH EDITION         .
*                      HAIR, BLACK, BABIN AND ANDERSON                     .
*                                                                                                      .
*                        CHAPTER 5 -- MULTIPLE REGRESSION                  .
*                                        SPSS SYNTAX                                       .
* ******************************************************************************** .
* ******************************************************************************************** .
* NOTE -- INSTRUCTIONS FOR DOWNLOADING AND EXECUTING                    .
*              SPSS MACROS FOR TWO RELATIVE IMPORTANCE MEASURES --  .
*              COMMONALITY AND RELATIVE WEIGHTS -- ARE AVAILABLE AT     .

*    Kraha, A., Turner, H., Nimon, K., Zientek, L., & Henson, R. (2012). 
*     Tools to support interpreting multiple regression in the face of multicollinearity. 
*     Frontiers in Psychology, 3, 44.
* ********************************************************************************************* .

* ************************************************************************************* .
*  NOTE -- THE SPSS OUTPUT IS SUPPLEMENTED BY SAS OUTPUT    .
*               IN SEVERAL SPECIFIC AREAS:                                            .
*                  -- HCSE ESTIMATE                                                            .
*                  -- GRAPHICAL DIAGNOSTICS -- LEVERAGE VS RESIDUAL .
*                  -- ALL POSSIBLE SUBSETS REGRESSION                         .
*                  -- MACROS FOR RELATIVE IMPORTANCE                          .
*                         -- DOMINANCE ANALYSIS                                           .
*                         -- RELATIVE WEIGHTS                                                .
*               PLEASE REFER TO THE ASSOCIATED SAS SYNTAX             .
*               AND OUTPUT FILES FOR THIS MATERIAL                              .
* ************************************************************************************* .


* ****************************************** .
* GET HBAT DATAFILE                     .
* ****************************************** . 

GET FILE = 'HBAT.SAV' .

* ********************************************************.
* BASIC STEPWISE REGRESSION ANALYSIS   .
* ******************************************************* .

TITLE 'STEPWISE REGRESSION' .

REGRESSION
  /DESCRIPTIVES MEAN STDDEV CORR SIG N
  /MISSING LISTWISE
  /STATISTICS COEFF OUTS CI(95) R ANOVA COLLIN TOL CHANGE ZPP
  /CRITERIA=PIN(.05) POUT(.10) CIN(95)
  /NOORIGIN 
  /DEPENDENT x19
  /METHOD=STEPWISE x6 x7 x8 x9 x10 x11 x12 x13 x14 x15 x16 x17 x18
  /PARTIALPLOT ALL
  /SCATTERPLOT=(*ZRESID ,*ZPRED)
  /RESIDUALS HISTOGRAM(ZRESID) NORMPROB(ZRESID)
  /SAVE PRED ZPRED ADJPRED SEPRED MAHAL COOK LEVER MCIN ICIN RESID ZRESID SRESID DRESID SDRESID 
    DFBETA SDBETA DFFIT SDFIT COVRATIO.


* **************************************************************** .
* USE CONFIRMATORY ESTIMATION PROCEDURE      .
* **************************************************************** .
TITLE 'CONFIRMATORY REGRESSION ANALYSIS' .

REGRESSION
  /DESCRIPTIVES MEAN STDDEV CORR SIG N
  /MISSING LISTWISE
  /STATISTICS COEFF OUTS R ANOVA COLLIN TOL CHANGE ZPP
  /CRITERIA=PIN(.05) POUT(.10)
  /NOORIGIN
  /DEPENDENT x19
  /METHOD=ENTER  x6 x7 x8 x9 x10 x11 x12 x13 x14 x15 x16 x17 x18
  /SCATTERPLOT=(*ZRESID ,*ZPRED )
  /RESIDUALS HIST(ZRESID) NORM(ZRESID)
  /CASEWISE PLOT(ZRESID) OUTLIERS(3) .



* *************************************** .
* ADD NONMETRIC VARIABLE X3 .
* *************************************** .


TITLE 'STEPWISE REGRESSION WITH NONMETRIC VARIABLE' .

REGRESSION
  /DESCRIPTIVES MEAN STDDEV CORR SIG N
  /MISSING LISTWISE
  /STATISTICS COEFF OUTS R ANOVA COLLIN TOL CHANGE ZPP
  /CRITERIA=PIN(.05) POUT(.10)
  /NOORIGIN
  /DEPENDENT x19
  /METHOD=STEPWISE x6 x7 x8 x9 x10 x11 x12 x13 x14 x15 x16 x17 x18 X3 
  /SCATTERPLOT=(*ZRESID ,*ZPRED )
  /RESIDUALS HIST(ZRESID) NORM(ZRESID)
  /CASEWISE PLOT(ZRESID) OUTLIERS(3) .



* ***************************************************************************** .
* INCLUDE TRANSFORMED VARIABLES TO ASSESS IMPACT     .
* TRANSFORM VARIABLES AS INDICATED IN CHAPTER 2           .
* ***************************************************************************** .

COMPUTE TX6 = X6 * X6 .
COMPUTE TX7 = LG10(X7) .
COMPUTE TX13 = X13 * X13 * X13 .
COMPUTE TX16 = X16 * X16 .
COMPUTE TX17 = 1 / X17 .

* *************************************************************************************** .
* SUBSTITUTE TRANSFORMED VARIABLES FOR ORIGNAL VARIABLES   .
* *************************************************************************************** .

TITLE 'IMPACT OF SUBSTITUTING TRANSFORMED VARIABLES' .

REGRESSION
  /DESCRIPTIVES MEAN STDDEV CORR SIG N
  /MISSING LISTWISE
  /STATISTICS COEFF OUTS R ANOVA COLLIN TOL CHANGE ZPP
  /CRITERIA=PIN(.05) POUT(.10)
  /NOORIGIN
  /DEPENDENT x19
  /METHOD=STEPWISE TX6 TX7 X8 X9 X10 X11 X12 TX13 X14 X15 TX16 TX17 x18
  /PARTIALPLOT ALL
  /SCATTERPLOT=(*ZRESID ,*ZPRED )
  /RESIDUALS HIST(ZRESID) NORM(ZRESID)
  /CASEWISE PLOT(ZRESID) OUTLIERS(3) .


* ********************************************************************************************************* .
* SPLIT SAMPLE AND ESTIMATE REGRESSION MODELS FOR EACH SUB-SAMPLE .
* MERGE 50/50 SPLIT VARIABLE FROM FILE HBAT_SPLITS                                            .
* ********************************************************************************************************* .

TITLE 'SPLIT-SAMPLE VALIDATION' .

SORT CASES BY ID .

MATCH FILES FILE = *
  /TABLE = 'HBAT_SPLITS.SAV'
  /BY ID .
SORT CASES BY SPLIT50 (A) .
EXECUTE .
SPLIT FILE BY SPLIT50 .

REGRESSION
  /DESCRIPTIVES MEAN STDDEV CORR SIG N
  /MISSING LISTWISE
  /STATISTICS COEFF OUTS R ANOVA COLLIN TOL CHANGE ZPP
  /CRITERIA=PIN(.05) POUT(.10)
  /NOORIGIN
  /DEPENDENT x19
  /METHOD=STEPWISE x6 x7 x8 x9 x10 x11 x12 x13 x14 x15 x16 x17 x18
  /SCATTERPLOT=(*ZRESID ,*ZPRED ) .

SPLIT FILE OFF .

* ************************************************************************************************************************ .
* ESTIMATE STEPWISE AND CONFIRMATORY REGRESSION MODELS USING SUMMATED SCALES .
* ************************************************************************************************************************ .

* ******************************************************************************** .
* COMPUTE SUMMATED SCALES FROM RESULTS OF CHAPTER 3 .
* ******************************************************************************** .
COMPUTE REV_X13 = 10 - X13 .
EXECUTE .

COMPUTE FAC_1 = MEAN (X9, X18, X16). 
COMPUTE FAC_2 = MEAN (X12 , X7, X10) .
COMPUTE FAC_3 = MEAN (X8, X14) .
COMPUTE FAC_4 = MEAN (X6, REV_X13) .
EXECUTE .

* ****************************************************************** .
* STEPWISE REGRESSION WITH SUMMATED SCALES .
* ****************************************************************** .

TITLE 'STEPWISE REGRESSION WITH SUMMATED SCALES' .

REGRESSION
  /DESCRIPTIVES MEAN STDDEV CORR SIG N
  /MISSING LISTWISE
  /STATISTICS COEFF OUTS CI(95) R ANOVA COLLIN TOL CHANGE ZPP
  /CRITERIA=PIN(.05) POUT(.10) CIN(95)
  /NOORIGIN 
  /DEPENDENT x19
  /METHOD=STEPWISE  FAC_1 FAC_2 FAC_3 FAC_4  X11, X15, X17
  /PARTIALPLOT ALL
  /SCATTERPLOT=(*ZRESID ,*ZPRED)
  /RESIDUALS HISTOGRAM(ZRESID) NORMPROB(ZRESID)
  /SAVE PRED ZPRED ADJPRED SEPRED MAHAL COOK LEVER MCIN ICIN RESID ZRESID SRESID DRESID SDRESID 
    DFBETA SDBETA DFFIT SDFIT COVRATIO.


* ************************************************************************* .
* CONFIRMATORY REGRESSION WITH SUMMATED SCALES .
* ************************************************************************* .

TITLE 'CONFIRMATORY REGRESSION WITH SUMMATED SCALES' .

REGRESSION
  /DESCRIPTIVES MEAN STDDEV CORR SIG N
  /MISSING LISTWISE
  /STATISTICS COEFF OUTS CI(95) R ANOVA COLLIN TOL CHANGE ZPP
  /CRITERIA=PIN(.05) POUT(.10) CIN(95)
  /NOORIGIN 
  /DEPENDENT x19
  /METHOD=ENTER  FAC_1 FAC_2 FAC_3 FAC_4  X11, X15, X17
  /PARTIALPLOT ALL
  /SCATTERPLOT=(*ZRESID ,*ZPRED)
  /RESIDUALS HISTOGRAM(ZRESID) NORMPROB(ZRESID)
  /SAVE PRED ZPRED ADJPRED SEPRED MAHAL COOK LEVER MCIN ICIN RESID ZRESID SRESID DRESID SDRESID 
    DFBETA SDBETA DFFIT SDFIT COVRATIO.

* *********************************************************************************************************** .
* STEPWISE REGRESSION MODEL WITH THREE INFLUENTIAL OBERVATIONS DELETED .
* *********************************************************************************************************** .

TITLE 'STEPWISE MODEL WITH THREE INFLUENTIAL OBERVATIONS DELETED' .

TEMPORARY .
SELECT IF (( ID NE 20) AND (ID NE 35) AND (ID NE 71) ).
REGRESSION
  /DESCRIPTIVES MEAN STDDEV CORR SIG N
  /MISSING LISTWISE
  /STATISTICS COEFF OUTS CI(95) R ANOVA COLLIN TOL CHANGE ZPP
  /CRITERIA=PIN(.05) POUT(.10) CIN(95)
  /NOORIGIN 
  /DEPENDENT x19
  /METHOD=STEPWISE x6 x7 x8 x9 x10 x11 x12 x13 x14 x15 x16 x17 x18
  /PARTIALPLOT ALL
  /SCATTERPLOT=(*ZRESID ,*ZPRED)
  /RESIDUALS HISTOGRAM(ZRESID) NORMPROB(ZRESID) .
 






