﻿* Encoding: UTF-8.


* ******************************************************************************** .
*                         SUPPLEMENTAL MATERIALS FOR                        .
*              MULTIVARIATE DATA ANALYSIS -- EIGHTH EDITION         .
*                      HAIR, BLACK, BABIN AND ANDERSON                     .
*                                                                                                      .
*                  CHAPTER 6 -- MANOVA -- EXTENDING ANOVA             .
*                                        SPSS SYNTAX                                       .
* ******************************************************************************** .

* NOTE -- EXAMPLES OF THE PROCESS MACRO ARE ILLUSTRATED .
*    AT THE END OF THIS FILE, BUT ACTUAL EXECUTION OF THE       .
*    PROCESS MACRO IS PERFORMED THROUGH A MENU SYSTEM .
*    .
*   DOCUMENTATION IS PROVIDED ALONG WITH MODEL SPECIFICATIONS .

* *********************************************************************************** .


* READ HBAT 200 FILE .

GET FILE = 'HBAT_200.SAV' .

* ************************************************ .
* CORRELATIONS OF OUTCOMES .
* ************************************************ .

TITLE 'CORRELATIONS OF OUTCOME MEASURES' .

CORRELATIONS
  /VARIABLES=x19 x20 x21
  /PRINT=TWOTAIL NOSIG
  /MISSING=PAIRWISE.

* ************************************************ .
* EXAMPLE 1 -- X5 BY X19 X20 X21 .
* ************************************************ .

TITLE 'EXAMPLE 1 -- X5 BY X19 X20 X21' .

GLM x19 x20 x21 BY x5
  /METHOD=SSTYPE(3)
  /INTERCEPT=INCLUDE
  /PLOT=PROFILE(x5)
  /EMMEANS=TABLES(x5) 
  /PRINT=DESCRIPTIVE ETASQ OPOWER PARAMETER HOMOGENEITY
  /PLOT=SPREADLEVEL 
  /CRITERIA=ALPHA(.05)
  /DESIGN= x5.

* ********************************************************************************************************************************* .
* EXAMPLE 1 -- STEPDOWN TESTS PERFORMED WITH DIFFERENT ORDERING OF OUTCOME MEASURES .
* ********************************************************************************************************************************* .

SUBTITLE 'EXAMPLE 1 -- STEPDOWN TESTS -- X19, X20, X21' .

MANOVA X19 X20 X21 BY X5 (0,1) /PRINT SIGNIF(STEPDOWN) /DISCRIM ALL .

SUBTITLE 'EXAMPLE 1 -- STEPDOWN TESTS -- X21, X20, X19' .

MANOVA X21 X20 X19  BY X5 (0,1) /PRINT SIGNIF(STEPDOWN) /DISCRIM ALL .

* ******************************************* .
* EXAMPLE 2 -- X1 BY X19 X20 X21 .
* ******************************************* .

TITLE 'EXAMPLE 2 -- X1 BY X19 X20 X21' .


GLM x19 x20 x21 BY x1
  /METHOD=SSTYPE(3)
  /INTERCEPT=INCLUDE
  /PLOT=PROFILE(x1)
  /EMMEANS=TABLES(x1) 
  /PRINT=DESCRIPTIVE ETASQ OPOWER PARAMETER HOMOGENEITY
  /PLOT=SPREADLEVEL 
  /CRITERIA=ALPHA(.05)
  /DESIGN= x1.


* ********************************************************************************************************************************* .
* EXAMPLE 2 -- STEPDOWN TESTS PERFORMED WITH DIFFERENT ORDERING OF OUTCOME MEASURES .
* ********************************************************************************************************************************* .

SUBTITLE 'EXAMPLE 2 -- STEPDOWN TESTS -- X19, X20, X21' .

MANOVA X19 X20 X21 BY X1 (1,3)  /PRINT SIGNIF(STEPDOWN) /DISCRIM ALL .

SUBTITLE 'EXAMPLE 2 -- STEPDOWN TESTS -- X21, X20, X19' .

MANOVA X21 X20 X19  BY X1 (1,3) /PRINT SIGNIF(STEPDOWN) /DISCRIM ALL .



* ************************************************* .
* EXAMPLE 3 -- X1 X5  BY X19 X20 X21 .
* ************************************************* .

TITLE 'EXAMPLE 3 -- X1 X5  BY X19 X20 X21' .


GLM x19 x20 x21 BY x5 x1
  /METHOD=SSTYPE(3)
  /INTERCEPT=INCLUDE
  /PLOT=PROFILE(x5*x1)
  /EMMEANS=TABLES(x5) 
  /EMMEANS=TABLES(x1) 
  /EMMEANS=TABLES(x5*x1) 
  /PRINT=DESCRIPTIVE ETASQ OPOWER PARAMETER HOMOGENEITY
  /PLOT=SPREADLEVEL 
  /CRITERIA=ALPHA(.05)
  /DESIGN= x5 x1 x5*x1.



* ********************************************************************************************************************************* .
* EXAMPLE 3 -- STEPDOWN TESTS PERFORMED WITH DIFFERENT ORDERING OF OUTCOME MEASURES .
* ********************************************************************************************************************************* .

SUBTITLE 'EXAMPLE 3 -- STEPDOWN TESTS -- X19, X20, X21' .

MANOVA X19 X20 X21 BY X1 (1,3) X5 (0 ,1)   /PRINT SIGNIF(STEPDOWN) /DISCRIM ALL .

SUBTITLE 'EXAMPLE 3 -- STEPDOWN TESTS -- X21, X20, X19' .

MANOVA X21 X20 X19  BY X1 (1,3) X5 (0, 1)  /PRINT SIGNIF(STEPDOWN) /DISCRIM ALL .



* ******************************************************************************************************* .
* EXAMPLE 4 -- USING THE PROCESS MACRO FOR MEDIATION AND MODERATION .
* ******************************************************************************************************* .

* ********************************************************************************************************************************* .

*  THE PROCESS MACRO IS DISTRIBUTED BY DR. ANDREW HAYES                       .
*       WWW.PROCESSMACRO.ORG                                                                         .
*                                                                                                                               .
*  INTERESTED USERS ARE ENCOURAGED TO DOWNLOAD THE LATEST VERSION .
*  OF THE PROCESS MACRO AS IMPROVEMENTS ARE BEING MADE CONTINUALLY .
*                                                                                                                               .
*  THE MODERATION AND MEDIATON EXAMPLES EMPLOY THE FOLOWING MODEL SPECIFICATIONS .
*                                                                                                                                 .

* MODERATION -- RUN PROCESS FOR EACH DV  -- MODEL 1    .
*    DV = X19, X20, X21  .
*    IV = x5  .
*    M = X3  .


* MEDIATION -- RUN PROCESS FOR EACH DV -- MODEL4 .
*    DV = X19, X20, X21  .
*    IV = x5  .
*    M = X22  .


*  OUTPUT FOR THESE MODELS ARE GIVEN IN THE EXAMPLE OUTPUT FILE  .

* ********************************************************************************************************************************* .








