﻿* Encoding: UTF-8.




* ******************************************************************************** .
*                         SUPPLEMENTAL MATERIALS FOR                        .
*              MULTIVARIATE DATA ANALYSIS -- EIGHTH EDITION         .
*                      HAIR, BLACK, BABIN AND ANDERSON                     .
*                                                                                                      .
*                        CHAPTER 8 -- LOGISTIC REGRESSION:                  .
*            REGRESSION WITH A BINARY DEPENDENT VARIABLE      .
*                                        SPSS SYNTAX                                        .
* ********************************************************************************  .


* ************************************************************************************* .
*  NOTE -- THE SPSS OUTPUT IS SUPPLEMENTED BY SAS OUTPUT .
*               IN SEVERAL SPECIFIC AREAS:                                         .
*                  -- SELECTING CUTOFF VALUES                                      .
*                  -- GRAPHICAL CASEWISE DIAGNOSTICS                        .
*                  -- GRAPHICAL PLOTS OF PARAMETER ESTIMATES       .

*               PLEASE REFER TO THE ASSOCIATED SAS SYNTAX         .
*               AND OUTPUT FILES FOR THIS MATERIAL                           .
* ************************************************************************************* .



TITLE 'CHAPTER 8 -- LOGISTIC REGRESSION' .

* *********************************************************************** .
* READ HBAT DATASET, SELECT 60 OBSERVATIONS FOR  .
*            UNIVARIATE MODEL ESTIMATION                           .
* *********************************************************************** .

GET FILE = 'HBAT.SAV' .

MATCH FILES FILE = *
  /TABLE = 'HBAT_SPLITS.SAV'
  /BY ID .

SELECT IF (SPLIT60 EQ 0) .

* ************************************************************************************** .
* UNIVARIATE ANALYSIS OF INDEPENDENT VARIABLES                        .
*   UNIVARIATE MODEL FOR EACH INDEPENDENT VARIABLE                .
*    -- REPEAT ANALYSIS BELOW FOR EACH INDEPENDENT VARIABLE .
* ************************************************************************************** .

TITLE 'UNIVARIATE LOGISTIC REGRESSION' .
SUBTITLE 'ESTIMATE FOR EACH INDEPENDENT VARIABLE' .
SUBTITLE 'X6' .


LOGISTIC REGRESSION VAR=x4 
 /method = enter x6 .





 ******************************************** .
GET FILE = 'HBAT.SAV' .

MATCH FILES FILE = *
  /TABLE = 'HBAT_SPLITS.SAV'
  /BY ID .
EXECUTE .

* *****************************************************************************************************  .
* ESTIMATE STEPWISE MODEL FOR ESTIMATION AND VALIDATION SUBSAMPLES   .
* ****************************************************************************************************** .

TITLE ' STEPWISE LOGISTIC REGRESSION' .

LOGISTIC REGRESSION VAR=x4
  /SELECT split60 EQ 0
  /METHOD=FSTEP(WALD) x6 x7 x8 x9 x10 x11 x12 x13 x14 x15 x16 x17 x18
  /SAVE PRED PGROUP COOK LEVER DFBETA RESID LRESID SRESID ZRESID
  /CLASSPLOT /CASEWISE OUTLIER(2)
  /PRINT=GOODFIT CORR ITER(1) CI(95)
  /CRITERIA PIN(.05) POUT(.10) ITERATE(100) CUT(.5) .


* ********************************************************************************* .
* SELECT ESTIMATION SUBSAMPLE FOR FOLLOWING ANALYSES  .
* ********************************************************************************* .

SELECT IF (SPLIT60 EQ 0) .

* ************************************************************************************ .
* ESTIMATE MULTIPLE CUTOFF VALUES .
*   -- CAN BE ESTIMATED FOR ANY CUTOFF VALUE DESIRED .
*   -- VALUES OF 0.0 AND 0.1 ILLUSTRATED HERE                   .
*   -- NOTE SAS PROCEDURE CAN ESTIMATE ALL CUUTOFF VALUES .
*         IN A SINGLE ANALYSIS -- SEE SAS OUTPUT .
* ************************************************************************************ .

TITLE 'ESTIMATION OF MULTIPLE CUTOFF VALUES' .
SUBTITLE 'CUTOFF VALUE -- 0.0' .

LOGISTIC REGRESSION VARIABLES x4
  /METHOD=ENTER x13 x17 
  /CRITERIA= CUT(0.0).

SUBTITLE 'CUTOFF VALUE -- 0.1' .

LOGISTIC REGRESSION VARIABLES x4
  /METHOD=ENTER x13 x17 
  /CRITERIA= CUT(0.1).



* ********************************************
* ROC CURVE  .
* ******************************************** .

TITLE 'ROC CURVE' .

ROC x13 x17 PRE_1 BY x4 (1)
  /PLOT=CURVE(REFERENCE)
  /PRINT= COORDINATES
  /CRITERIA=CUTOFF(INCLUDE) TESTPOS(LARGE) DISTRIBUTION(FREE) CI(95)
  /MISSING=EXCLUDE.


* ************************************************************************************ .
* TEST FOR NONLINEAR EFFECTS                                                       .
*    1. CREATE INTERACTION TERM                                                     
*    2. ESTIMATE UNIVARIATE MODEL WITH INTERACTION TERMS       .        .
*    2. ESTIMATE FINAL MODEL WITH INTERACTION TERMS                 .
* ************************************************************************************ .

* ******************************************************************************************************************* .
* FOLLOWING ANALYSIS FOR X6, CAN BE REPEATED FOR OTHER INDEPENDENT VARIABLES .
* ******************************************************************************************************************** .

COMPUTE X6_LOG = LN(X6) .
COMPUTE X6_INT = X6 * X6_LOG .
EXECUTE .

TITLE ' TESTING FOR NONLINEAR EFFECTS' .
SUBTITLE 'X6 -- TESTING UNIVARIATE MODEL WITH INTERACTION' .
LOGISTIC REGRESSION VAR=x4
 /method = enter x6 
 /METHOD = ENTER X6_INT .

SUBTITLE 'X6 -- ADDING INTERACTION TERM TO FINAL MODEL' .
LOGISTIC REGRESSION VAR=x4
 /method = enter X13 X17 
 /METHOD = ENTER X6 X6_INT .

